# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: src/ray/protobuf/node_manager.proto

import sys
_b=sys.version_info[0]<3 and (lambda x:x) or (lambda x:x.encode('latin1'))
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from . import common_pb2 as src_dot_ray_dot_protobuf_dot_common__pb2


DESCRIPTOR = _descriptor.FileDescriptor(
  name='src/ray/protobuf/node_manager.proto',
  package='ray.rpc',
  syntax='proto3',
  serialized_options=None,
  serialized_pb=_b('\n#src/ray/protobuf/node_manager.proto\x12\x07ray.rpc\x1a\x1dsrc/ray/protobuf/common.proto\"v\n\x19RequestWorkerLeaseRequest\x12\x36\n\rresource_spec\x18\x01 \x01(\x0b\x32\x11.ray.rpc.TaskSpecR\x0cresourceSpec\x12!\n\x0c\x62\x61\x63klog_size\x18\x02 \x01(\x03R\x0b\x62\x61\x63klogSize\"\xfd\x01\n\x17RequestWorkerLeaseReply\x12\x37\n\x0eworker_address\x18\x01 \x01(\x0b\x32\x10.ray.rpc.AddressR\rworkerAddress\x12G\n\x17retry_at_raylet_address\x18\x02 \x01(\x0b\x32\x10.ray.rpc.AddressR\x14retryAtRayletAddress\x12\x44\n\x10resource_mapping\x18\x03 \x03(\x0b\x32\x19.ray.rpc.ResourceMapEntryR\x0fresourceMapping\x12\x1a\n\x08\x63\x61nceled\x18\x04 \x01(\x08R\x08\x63\x61nceled\"Q\n\x1dPrepareBundleResourcesRequest\x12\x30\n\x0b\x62undle_spec\x18\x01 \x01(\x0b\x32\x0f.ray.rpc.BundleR\nbundleSpec\"7\n\x1bPrepareBundleResourcesReply\x12\x18\n\x07success\x18\x01 \x01(\x08R\x07success\"P\n\x1c\x43ommitBundleResourcesRequest\x12\x30\n\x0b\x62undle_spec\x18\x01 \x01(\x0b\x32\x0f.ray.rpc.BundleR\nbundleSpec\"\x1c\n\x1a\x43ommitBundleResourcesReply\"P\n\x1c\x43\x61ncelResourceReserveRequest\x12\x30\n\x0b\x62undle_spec\x18\x01 \x01(\x0b\x32\x0f.ray.rpc.BundleR\nbundleSpec\"\x1c\n\x1a\x43\x61ncelResourceReserveReply\"\x80\x01\n\x13ReturnWorkerRequest\x12\x1f\n\x0bworker_port\x18\x01 \x01(\x05R\nworkerPort\x12\x1b\n\tworker_id\x18\x02 \x01(\x0cR\x08workerId\x12+\n\x11\x64isconnect_worker\x18\x03 \x01(\x08R\x10\x64isconnectWorker\"\x13\n\x11ReturnWorkerReply\"H\n\x1bReleaseUnusedWorkersRequest\x12)\n\x11worker_ids_in_use\x18\x01 \x03(\x0cR\x0eworkerIdsInUse\"\x1b\n\x19ReleaseUnusedWorkersReply\"3\n\x18\x43\x61ncelWorkerLeaseRequest\x12\x17\n\x07task_id\x18\x01 \x01(\x0cR\x06taskId\"2\n\x16\x43\x61ncelWorkerLeaseReply\x12\x18\n\x07success\x18\x01 \x01(\x08R\x07success\"k\n\x13PinObjectIDsRequest\x12\x35\n\rowner_address\x18\x01 \x01(\x0b\x32\x10.ray.rpc.AddressR\x0cownerAddress\x12\x1d\n\nobject_ids\x18\x02 \x03(\x0cR\tobjectIds\"\x13\n\x11PinObjectIDsReply\"E\n\x13GetNodeStatsRequest\x12.\n\x13include_memory_info\x18\x01 \x01(\x08R\x11includeMemoryInfo\"\x9e\x02\n\x11GetNodeStatsReply\x12\x46\n\x12\x63ore_workers_stats\x18\x01 \x03(\x0b\x32\x18.ray.rpc.CoreWorkerStatsR\x10\x63oreWorkersStats\x12.\n\tview_data\x18\x02 \x03(\x0b\x32\x11.ray.rpc.ViewDataR\x08viewData\x12\x1f\n\x0bnum_workers\x18\x03 \x01(\rR\nnumWorkers\x12<\n\x10infeasible_tasks\x18\x04 \x03(\x0b\x32\x11.ray.rpc.TaskSpecR\x0finfeasibleTasks\x12\x32\n\x0bready_tasks\x18\x05 \x03(\x0b\x32\x11.ray.rpc.TaskSpecR\nreadyTasks\"\x11\n\x0fGlobalGCRequest\"\x0f\n\rGlobalGCReply\"\x1f\n\x1d\x46ormatGlobalMemoryInfoRequest\"D\n\x1b\x46ormatGlobalMemoryInfoReply\x12%\n\x0ememory_summary\x18\x01 \x01(\tR\rmemorySummary\";\n\x1cRequestObjectSpillageRequest\x12\x1b\n\tobject_id\x18\x01 \x01(\x0cR\x08objectId\"6\n\x1aRequestObjectSpillageReply\x12\x18\n\x07success\x18\x01 \x01(\x08R\x07success\"T\n\x1bReleaseUnusedBundlesRequest\x12\x35\n\x0e\x62undles_in_use\x18\x01 \x03(\x0b\x32\x0f.ray.rpc.BundleR\x0c\x62undlesInUse\"\x1b\n\x19ReleaseUnusedBundlesReply2\xa8\t\n\x12NodeManagerService\x12Z\n\x12RequestWorkerLease\x12\".ray.rpc.RequestWorkerLeaseRequest\x1a .ray.rpc.RequestWorkerLeaseReply\x12H\n\x0cReturnWorker\x12\x1c.ray.rpc.ReturnWorkerRequest\x1a\x1a.ray.rpc.ReturnWorkerReply\x12`\n\x14ReleaseUnusedWorkers\x12$.ray.rpc.ReleaseUnusedWorkersRequest\x1a\".ray.rpc.ReleaseUnusedWorkersReply\x12\x66\n\x16PrepareBundleResources\x12&.ray.rpc.PrepareBundleResourcesRequest\x1a$.ray.rpc.PrepareBundleResourcesReply\x12\x63\n\x15\x43ommitBundleResources\x12%.ray.rpc.CommitBundleResourcesRequest\x1a#.ray.rpc.CommitBundleResourcesReply\x12\x63\n\x15\x43\x61ncelResourceReserve\x12%.ray.rpc.CancelResourceReserveRequest\x1a#.ray.rpc.CancelResourceReserveReply\x12W\n\x11\x43\x61ncelWorkerLease\x12!.ray.rpc.CancelWorkerLeaseRequest\x1a\x1f.ray.rpc.CancelWorkerLeaseReply\x12H\n\x0cPinObjectIDs\x12\x1c.ray.rpc.PinObjectIDsRequest\x1a\x1a.ray.rpc.PinObjectIDsReply\x12H\n\x0cGetNodeStats\x12\x1c.ray.rpc.GetNodeStatsRequest\x1a\x1a.ray.rpc.GetNodeStatsReply\x12<\n\x08GlobalGC\x12\x18.ray.rpc.GlobalGCRequest\x1a\x16.ray.rpc.GlobalGCReply\x12\x66\n\x16\x46ormatGlobalMemoryInfo\x12&.ray.rpc.FormatGlobalMemoryInfoRequest\x1a$.ray.rpc.FormatGlobalMemoryInfoReply\x12\x63\n\x15RequestObjectSpillage\x12%.ray.rpc.RequestObjectSpillageRequest\x1a#.ray.rpc.RequestObjectSpillageReply\x12`\n\x14ReleaseUnusedBundles\x12$.ray.rpc.ReleaseUnusedBundlesRequest\x1a\".ray.rpc.ReleaseUnusedBundlesReplyb\x06proto3')
  ,
  dependencies=[src_dot_ray_dot_protobuf_dot_common__pb2.DESCRIPTOR,])




_REQUESTWORKERLEASEREQUEST = _descriptor.Descriptor(
  name='RequestWorkerLeaseRequest',
  full_name='ray.rpc.RequestWorkerLeaseRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='resource_spec', full_name='ray.rpc.RequestWorkerLeaseRequest.resource_spec', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='resourceSpec', file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='backlog_size', full_name='ray.rpc.RequestWorkerLeaseRequest.backlog_size', index=1,
      number=2, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='backlogSize', file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=79,
  serialized_end=197,
)


_REQUESTWORKERLEASEREPLY = _descriptor.Descriptor(
  name='RequestWorkerLeaseReply',
  full_name='ray.rpc.RequestWorkerLeaseReply',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='worker_address', full_name='ray.rpc.RequestWorkerLeaseReply.worker_address', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='workerAddress', file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='retry_at_raylet_address', full_name='ray.rpc.RequestWorkerLeaseReply.retry_at_raylet_address', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='retryAtRayletAddress', file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='resource_mapping', full_name='ray.rpc.RequestWorkerLeaseReply.resource_mapping', index=2,
      number=3, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='resourceMapping', file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='canceled', full_name='ray.rpc.RequestWorkerLeaseReply.canceled', index=3,
      number=4, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='canceled', file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=200,
  serialized_end=453,
)


_PREPAREBUNDLERESOURCESREQUEST = _descriptor.Descriptor(
  name='PrepareBundleResourcesRequest',
  full_name='ray.rpc.PrepareBundleResourcesRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='bundle_spec', full_name='ray.rpc.PrepareBundleResourcesRequest.bundle_spec', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='bundleSpec', file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=455,
  serialized_end=536,
)


_PREPAREBUNDLERESOURCESREPLY = _descriptor.Descriptor(
  name='PrepareBundleResourcesReply',
  full_name='ray.rpc.PrepareBundleResourcesReply',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='success', full_name='ray.rpc.PrepareBundleResourcesReply.success', index=0,
      number=1, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='success', file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=538,
  serialized_end=593,
)


_COMMITBUNDLERESOURCESREQUEST = _descriptor.Descriptor(
  name='CommitBundleResourcesRequest',
  full_name='ray.rpc.CommitBundleResourcesRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='bundle_spec', full_name='ray.rpc.CommitBundleResourcesRequest.bundle_spec', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='bundleSpec', file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=595,
  serialized_end=675,
)


_COMMITBUNDLERESOURCESREPLY = _descriptor.Descriptor(
  name='CommitBundleResourcesReply',
  full_name='ray.rpc.CommitBundleResourcesReply',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=677,
  serialized_end=705,
)


_CANCELRESOURCERESERVEREQUEST = _descriptor.Descriptor(
  name='CancelResourceReserveRequest',
  full_name='ray.rpc.CancelResourceReserveRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='bundle_spec', full_name='ray.rpc.CancelResourceReserveRequest.bundle_spec', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='bundleSpec', file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=707,
  serialized_end=787,
)


_CANCELRESOURCERESERVEREPLY = _descriptor.Descriptor(
  name='CancelResourceReserveReply',
  full_name='ray.rpc.CancelResourceReserveReply',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=789,
  serialized_end=817,
)


_RETURNWORKERREQUEST = _descriptor.Descriptor(
  name='ReturnWorkerRequest',
  full_name='ray.rpc.ReturnWorkerRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='worker_port', full_name='ray.rpc.ReturnWorkerRequest.worker_port', index=0,
      number=1, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='workerPort', file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='worker_id', full_name='ray.rpc.ReturnWorkerRequest.worker_id', index=1,
      number=2, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=_b(""),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='workerId', file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='disconnect_worker', full_name='ray.rpc.ReturnWorkerRequest.disconnect_worker', index=2,
      number=3, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='disconnectWorker', file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=820,
  serialized_end=948,
)


_RETURNWORKERREPLY = _descriptor.Descriptor(
  name='ReturnWorkerReply',
  full_name='ray.rpc.ReturnWorkerReply',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=950,
  serialized_end=969,
)


_RELEASEUNUSEDWORKERSREQUEST = _descriptor.Descriptor(
  name='ReleaseUnusedWorkersRequest',
  full_name='ray.rpc.ReleaseUnusedWorkersRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='worker_ids_in_use', full_name='ray.rpc.ReleaseUnusedWorkersRequest.worker_ids_in_use', index=0,
      number=1, type=12, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='workerIdsInUse', file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=971,
  serialized_end=1043,
)


_RELEASEUNUSEDWORKERSREPLY = _descriptor.Descriptor(
  name='ReleaseUnusedWorkersReply',
  full_name='ray.rpc.ReleaseUnusedWorkersReply',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1045,
  serialized_end=1072,
)


_CANCELWORKERLEASEREQUEST = _descriptor.Descriptor(
  name='CancelWorkerLeaseRequest',
  full_name='ray.rpc.CancelWorkerLeaseRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='task_id', full_name='ray.rpc.CancelWorkerLeaseRequest.task_id', index=0,
      number=1, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=_b(""),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='taskId', file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1074,
  serialized_end=1125,
)


_CANCELWORKERLEASEREPLY = _descriptor.Descriptor(
  name='CancelWorkerLeaseReply',
  full_name='ray.rpc.CancelWorkerLeaseReply',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='success', full_name='ray.rpc.CancelWorkerLeaseReply.success', index=0,
      number=1, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='success', file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1127,
  serialized_end=1177,
)


_PINOBJECTIDSREQUEST = _descriptor.Descriptor(
  name='PinObjectIDsRequest',
  full_name='ray.rpc.PinObjectIDsRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='owner_address', full_name='ray.rpc.PinObjectIDsRequest.owner_address', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='ownerAddress', file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='object_ids', full_name='ray.rpc.PinObjectIDsRequest.object_ids', index=1,
      number=2, type=12, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='objectIds', file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1179,
  serialized_end=1286,
)


_PINOBJECTIDSREPLY = _descriptor.Descriptor(
  name='PinObjectIDsReply',
  full_name='ray.rpc.PinObjectIDsReply',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1288,
  serialized_end=1307,
)


_GETNODESTATSREQUEST = _descriptor.Descriptor(
  name='GetNodeStatsRequest',
  full_name='ray.rpc.GetNodeStatsRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='include_memory_info', full_name='ray.rpc.GetNodeStatsRequest.include_memory_info', index=0,
      number=1, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='includeMemoryInfo', file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1309,
  serialized_end=1378,
)


_GETNODESTATSREPLY = _descriptor.Descriptor(
  name='GetNodeStatsReply',
  full_name='ray.rpc.GetNodeStatsReply',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='core_workers_stats', full_name='ray.rpc.GetNodeStatsReply.core_workers_stats', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='coreWorkersStats', file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='view_data', full_name='ray.rpc.GetNodeStatsReply.view_data', index=1,
      number=2, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='viewData', file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='num_workers', full_name='ray.rpc.GetNodeStatsReply.num_workers', index=2,
      number=3, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='numWorkers', file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='infeasible_tasks', full_name='ray.rpc.GetNodeStatsReply.infeasible_tasks', index=3,
      number=4, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='infeasibleTasks', file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='ready_tasks', full_name='ray.rpc.GetNodeStatsReply.ready_tasks', index=4,
      number=5, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='readyTasks', file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1381,
  serialized_end=1667,
)


_GLOBALGCREQUEST = _descriptor.Descriptor(
  name='GlobalGCRequest',
  full_name='ray.rpc.GlobalGCRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1669,
  serialized_end=1686,
)


_GLOBALGCREPLY = _descriptor.Descriptor(
  name='GlobalGCReply',
  full_name='ray.rpc.GlobalGCReply',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1688,
  serialized_end=1703,
)


_FORMATGLOBALMEMORYINFOREQUEST = _descriptor.Descriptor(
  name='FormatGlobalMemoryInfoRequest',
  full_name='ray.rpc.FormatGlobalMemoryInfoRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1705,
  serialized_end=1736,
)


_FORMATGLOBALMEMORYINFOREPLY = _descriptor.Descriptor(
  name='FormatGlobalMemoryInfoReply',
  full_name='ray.rpc.FormatGlobalMemoryInfoReply',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='memory_summary', full_name='ray.rpc.FormatGlobalMemoryInfoReply.memory_summary', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='memorySummary', file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1738,
  serialized_end=1806,
)


_REQUESTOBJECTSPILLAGEREQUEST = _descriptor.Descriptor(
  name='RequestObjectSpillageRequest',
  full_name='ray.rpc.RequestObjectSpillageRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='object_id', full_name='ray.rpc.RequestObjectSpillageRequest.object_id', index=0,
      number=1, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=_b(""),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='objectId', file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1808,
  serialized_end=1867,
)


_REQUESTOBJECTSPILLAGEREPLY = _descriptor.Descriptor(
  name='RequestObjectSpillageReply',
  full_name='ray.rpc.RequestObjectSpillageReply',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='success', full_name='ray.rpc.RequestObjectSpillageReply.success', index=0,
      number=1, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='success', file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1869,
  serialized_end=1923,
)


_RELEASEUNUSEDBUNDLESREQUEST = _descriptor.Descriptor(
  name='ReleaseUnusedBundlesRequest',
  full_name='ray.rpc.ReleaseUnusedBundlesRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='bundles_in_use', full_name='ray.rpc.ReleaseUnusedBundlesRequest.bundles_in_use', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='bundlesInUse', file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1925,
  serialized_end=2009,
)


_RELEASEUNUSEDBUNDLESREPLY = _descriptor.Descriptor(
  name='ReleaseUnusedBundlesReply',
  full_name='ray.rpc.ReleaseUnusedBundlesReply',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=2011,
  serialized_end=2038,
)

_REQUESTWORKERLEASEREQUEST.fields_by_name['resource_spec'].message_type = src_dot_ray_dot_protobuf_dot_common__pb2._TASKSPEC
_REQUESTWORKERLEASEREPLY.fields_by_name['worker_address'].message_type = src_dot_ray_dot_protobuf_dot_common__pb2._ADDRESS
_REQUESTWORKERLEASEREPLY.fields_by_name['retry_at_raylet_address'].message_type = src_dot_ray_dot_protobuf_dot_common__pb2._ADDRESS
_REQUESTWORKERLEASEREPLY.fields_by_name['resource_mapping'].message_type = src_dot_ray_dot_protobuf_dot_common__pb2._RESOURCEMAPENTRY
_PREPAREBUNDLERESOURCESREQUEST.fields_by_name['bundle_spec'].message_type = src_dot_ray_dot_protobuf_dot_common__pb2._BUNDLE
_COMMITBUNDLERESOURCESREQUEST.fields_by_name['bundle_spec'].message_type = src_dot_ray_dot_protobuf_dot_common__pb2._BUNDLE
_CANCELRESOURCERESERVEREQUEST.fields_by_name['bundle_spec'].message_type = src_dot_ray_dot_protobuf_dot_common__pb2._BUNDLE
_PINOBJECTIDSREQUEST.fields_by_name['owner_address'].message_type = src_dot_ray_dot_protobuf_dot_common__pb2._ADDRESS
_GETNODESTATSREPLY.fields_by_name['core_workers_stats'].message_type = src_dot_ray_dot_protobuf_dot_common__pb2._COREWORKERSTATS
_GETNODESTATSREPLY.fields_by_name['view_data'].message_type = src_dot_ray_dot_protobuf_dot_common__pb2._VIEWDATA
_GETNODESTATSREPLY.fields_by_name['infeasible_tasks'].message_type = src_dot_ray_dot_protobuf_dot_common__pb2._TASKSPEC
_GETNODESTATSREPLY.fields_by_name['ready_tasks'].message_type = src_dot_ray_dot_protobuf_dot_common__pb2._TASKSPEC
_RELEASEUNUSEDBUNDLESREQUEST.fields_by_name['bundles_in_use'].message_type = src_dot_ray_dot_protobuf_dot_common__pb2._BUNDLE
DESCRIPTOR.message_types_by_name['RequestWorkerLeaseRequest'] = _REQUESTWORKERLEASEREQUEST
DESCRIPTOR.message_types_by_name['RequestWorkerLeaseReply'] = _REQUESTWORKERLEASEREPLY
DESCRIPTOR.message_types_by_name['PrepareBundleResourcesRequest'] = _PREPAREBUNDLERESOURCESREQUEST
DESCRIPTOR.message_types_by_name['PrepareBundleResourcesReply'] = _PREPAREBUNDLERESOURCESREPLY
DESCRIPTOR.message_types_by_name['CommitBundleResourcesRequest'] = _COMMITBUNDLERESOURCESREQUEST
DESCRIPTOR.message_types_by_name['CommitBundleResourcesReply'] = _COMMITBUNDLERESOURCESREPLY
DESCRIPTOR.message_types_by_name['CancelResourceReserveRequest'] = _CANCELRESOURCERESERVEREQUEST
DESCRIPTOR.message_types_by_name['CancelResourceReserveReply'] = _CANCELRESOURCERESERVEREPLY
DESCRIPTOR.message_types_by_name['ReturnWorkerRequest'] = _RETURNWORKERREQUEST
DESCRIPTOR.message_types_by_name['ReturnWorkerReply'] = _RETURNWORKERREPLY
DESCRIPTOR.message_types_by_name['ReleaseUnusedWorkersRequest'] = _RELEASEUNUSEDWORKERSREQUEST
DESCRIPTOR.message_types_by_name['ReleaseUnusedWorkersReply'] = _RELEASEUNUSEDWORKERSREPLY
DESCRIPTOR.message_types_by_name['CancelWorkerLeaseRequest'] = _CANCELWORKERLEASEREQUEST
DESCRIPTOR.message_types_by_name['CancelWorkerLeaseReply'] = _CANCELWORKERLEASEREPLY
DESCRIPTOR.message_types_by_name['PinObjectIDsRequest'] = _PINOBJECTIDSREQUEST
DESCRIPTOR.message_types_by_name['PinObjectIDsReply'] = _PINOBJECTIDSREPLY
DESCRIPTOR.message_types_by_name['GetNodeStatsRequest'] = _GETNODESTATSREQUEST
DESCRIPTOR.message_types_by_name['GetNodeStatsReply'] = _GETNODESTATSREPLY
DESCRIPTOR.message_types_by_name['GlobalGCRequest'] = _GLOBALGCREQUEST
DESCRIPTOR.message_types_by_name['GlobalGCReply'] = _GLOBALGCREPLY
DESCRIPTOR.message_types_by_name['FormatGlobalMemoryInfoRequest'] = _FORMATGLOBALMEMORYINFOREQUEST
DESCRIPTOR.message_types_by_name['FormatGlobalMemoryInfoReply'] = _FORMATGLOBALMEMORYINFOREPLY
DESCRIPTOR.message_types_by_name['RequestObjectSpillageRequest'] = _REQUESTOBJECTSPILLAGEREQUEST
DESCRIPTOR.message_types_by_name['RequestObjectSpillageReply'] = _REQUESTOBJECTSPILLAGEREPLY
DESCRIPTOR.message_types_by_name['ReleaseUnusedBundlesRequest'] = _RELEASEUNUSEDBUNDLESREQUEST
DESCRIPTOR.message_types_by_name['ReleaseUnusedBundlesReply'] = _RELEASEUNUSEDBUNDLESREPLY
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

RequestWorkerLeaseRequest = _reflection.GeneratedProtocolMessageType('RequestWorkerLeaseRequest', (_message.Message,), {
  'DESCRIPTOR' : _REQUESTWORKERLEASEREQUEST,
  '__module__' : 'src.ray.protobuf.node_manager_pb2'
  # @@protoc_insertion_point(class_scope:ray.rpc.RequestWorkerLeaseRequest)
  })
_sym_db.RegisterMessage(RequestWorkerLeaseRequest)

RequestWorkerLeaseReply = _reflection.GeneratedProtocolMessageType('RequestWorkerLeaseReply', (_message.Message,), {
  'DESCRIPTOR' : _REQUESTWORKERLEASEREPLY,
  '__module__' : 'src.ray.protobuf.node_manager_pb2'
  # @@protoc_insertion_point(class_scope:ray.rpc.RequestWorkerLeaseReply)
  })
_sym_db.RegisterMessage(RequestWorkerLeaseReply)

PrepareBundleResourcesRequest = _reflection.GeneratedProtocolMessageType('PrepareBundleResourcesRequest', (_message.Message,), {
  'DESCRIPTOR' : _PREPAREBUNDLERESOURCESREQUEST,
  '__module__' : 'src.ray.protobuf.node_manager_pb2'
  # @@protoc_insertion_point(class_scope:ray.rpc.PrepareBundleResourcesRequest)
  })
_sym_db.RegisterMessage(PrepareBundleResourcesRequest)

PrepareBundleResourcesReply = _reflection.GeneratedProtocolMessageType('PrepareBundleResourcesReply', (_message.Message,), {
  'DESCRIPTOR' : _PREPAREBUNDLERESOURCESREPLY,
  '__module__' : 'src.ray.protobuf.node_manager_pb2'
  # @@protoc_insertion_point(class_scope:ray.rpc.PrepareBundleResourcesReply)
  })
_sym_db.RegisterMessage(PrepareBundleResourcesReply)

CommitBundleResourcesRequest = _reflection.GeneratedProtocolMessageType('CommitBundleResourcesRequest', (_message.Message,), {
  'DESCRIPTOR' : _COMMITBUNDLERESOURCESREQUEST,
  '__module__' : 'src.ray.protobuf.node_manager_pb2'
  # @@protoc_insertion_point(class_scope:ray.rpc.CommitBundleResourcesRequest)
  })
_sym_db.RegisterMessage(CommitBundleResourcesRequest)

CommitBundleResourcesReply = _reflection.GeneratedProtocolMessageType('CommitBundleResourcesReply', (_message.Message,), {
  'DESCRIPTOR' : _COMMITBUNDLERESOURCESREPLY,
  '__module__' : 'src.ray.protobuf.node_manager_pb2'
  # @@protoc_insertion_point(class_scope:ray.rpc.CommitBundleResourcesReply)
  })
_sym_db.RegisterMessage(CommitBundleResourcesReply)

CancelResourceReserveRequest = _reflection.GeneratedProtocolMessageType('CancelResourceReserveRequest', (_message.Message,), {
  'DESCRIPTOR' : _CANCELRESOURCERESERVEREQUEST,
  '__module__' : 'src.ray.protobuf.node_manager_pb2'
  # @@protoc_insertion_point(class_scope:ray.rpc.CancelResourceReserveRequest)
  })
_sym_db.RegisterMessage(CancelResourceReserveRequest)

CancelResourceReserveReply = _reflection.GeneratedProtocolMessageType('CancelResourceReserveReply', (_message.Message,), {
  'DESCRIPTOR' : _CANCELRESOURCERESERVEREPLY,
  '__module__' : 'src.ray.protobuf.node_manager_pb2'
  # @@protoc_insertion_point(class_scope:ray.rpc.CancelResourceReserveReply)
  })
_sym_db.RegisterMessage(CancelResourceReserveReply)

ReturnWorkerRequest = _reflection.GeneratedProtocolMessageType('ReturnWorkerRequest', (_message.Message,), {
  'DESCRIPTOR' : _RETURNWORKERREQUEST,
  '__module__' : 'src.ray.protobuf.node_manager_pb2'
  # @@protoc_insertion_point(class_scope:ray.rpc.ReturnWorkerRequest)
  })
_sym_db.RegisterMessage(ReturnWorkerRequest)

ReturnWorkerReply = _reflection.GeneratedProtocolMessageType('ReturnWorkerReply', (_message.Message,), {
  'DESCRIPTOR' : _RETURNWORKERREPLY,
  '__module__' : 'src.ray.protobuf.node_manager_pb2'
  # @@protoc_insertion_point(class_scope:ray.rpc.ReturnWorkerReply)
  })
_sym_db.RegisterMessage(ReturnWorkerReply)

ReleaseUnusedWorkersRequest = _reflection.GeneratedProtocolMessageType('ReleaseUnusedWorkersRequest', (_message.Message,), {
  'DESCRIPTOR' : _RELEASEUNUSEDWORKERSREQUEST,
  '__module__' : 'src.ray.protobuf.node_manager_pb2'
  # @@protoc_insertion_point(class_scope:ray.rpc.ReleaseUnusedWorkersRequest)
  })
_sym_db.RegisterMessage(ReleaseUnusedWorkersRequest)

ReleaseUnusedWorkersReply = _reflection.GeneratedProtocolMessageType('ReleaseUnusedWorkersReply', (_message.Message,), {
  'DESCRIPTOR' : _RELEASEUNUSEDWORKERSREPLY,
  '__module__' : 'src.ray.protobuf.node_manager_pb2'
  # @@protoc_insertion_point(class_scope:ray.rpc.ReleaseUnusedWorkersReply)
  })
_sym_db.RegisterMessage(ReleaseUnusedWorkersReply)

CancelWorkerLeaseRequest = _reflection.GeneratedProtocolMessageType('CancelWorkerLeaseRequest', (_message.Message,), {
  'DESCRIPTOR' : _CANCELWORKERLEASEREQUEST,
  '__module__' : 'src.ray.protobuf.node_manager_pb2'
  # @@protoc_insertion_point(class_scope:ray.rpc.CancelWorkerLeaseRequest)
  })
_sym_db.RegisterMessage(CancelWorkerLeaseRequest)

CancelWorkerLeaseReply = _reflection.GeneratedProtocolMessageType('CancelWorkerLeaseReply', (_message.Message,), {
  'DESCRIPTOR' : _CANCELWORKERLEASEREPLY,
  '__module__' : 'src.ray.protobuf.node_manager_pb2'
  # @@protoc_insertion_point(class_scope:ray.rpc.CancelWorkerLeaseReply)
  })
_sym_db.RegisterMessage(CancelWorkerLeaseReply)

PinObjectIDsRequest = _reflection.GeneratedProtocolMessageType('PinObjectIDsRequest', (_message.Message,), {
  'DESCRIPTOR' : _PINOBJECTIDSREQUEST,
  '__module__' : 'src.ray.protobuf.node_manager_pb2'
  # @@protoc_insertion_point(class_scope:ray.rpc.PinObjectIDsRequest)
  })
_sym_db.RegisterMessage(PinObjectIDsRequest)

PinObjectIDsReply = _reflection.GeneratedProtocolMessageType('PinObjectIDsReply', (_message.Message,), {
  'DESCRIPTOR' : _PINOBJECTIDSREPLY,
  '__module__' : 'src.ray.protobuf.node_manager_pb2'
  # @@protoc_insertion_point(class_scope:ray.rpc.PinObjectIDsReply)
  })
_sym_db.RegisterMessage(PinObjectIDsReply)

GetNodeStatsRequest = _reflection.GeneratedProtocolMessageType('GetNodeStatsRequest', (_message.Message,), {
  'DESCRIPTOR' : _GETNODESTATSREQUEST,
  '__module__' : 'src.ray.protobuf.node_manager_pb2'
  # @@protoc_insertion_point(class_scope:ray.rpc.GetNodeStatsRequest)
  })
_sym_db.RegisterMessage(GetNodeStatsRequest)

GetNodeStatsReply = _reflection.GeneratedProtocolMessageType('GetNodeStatsReply', (_message.Message,), {
  'DESCRIPTOR' : _GETNODESTATSREPLY,
  '__module__' : 'src.ray.protobuf.node_manager_pb2'
  # @@protoc_insertion_point(class_scope:ray.rpc.GetNodeStatsReply)
  })
_sym_db.RegisterMessage(GetNodeStatsReply)

GlobalGCRequest = _reflection.GeneratedProtocolMessageType('GlobalGCRequest', (_message.Message,), {
  'DESCRIPTOR' : _GLOBALGCREQUEST,
  '__module__' : 'src.ray.protobuf.node_manager_pb2'
  # @@protoc_insertion_point(class_scope:ray.rpc.GlobalGCRequest)
  })
_sym_db.RegisterMessage(GlobalGCRequest)

GlobalGCReply = _reflection.GeneratedProtocolMessageType('GlobalGCReply', (_message.Message,), {
  'DESCRIPTOR' : _GLOBALGCREPLY,
  '__module__' : 'src.ray.protobuf.node_manager_pb2'
  # @@protoc_insertion_point(class_scope:ray.rpc.GlobalGCReply)
  })
_sym_db.RegisterMessage(GlobalGCReply)

FormatGlobalMemoryInfoRequest = _reflection.GeneratedProtocolMessageType('FormatGlobalMemoryInfoRequest', (_message.Message,), {
  'DESCRIPTOR' : _FORMATGLOBALMEMORYINFOREQUEST,
  '__module__' : 'src.ray.protobuf.node_manager_pb2'
  # @@protoc_insertion_point(class_scope:ray.rpc.FormatGlobalMemoryInfoRequest)
  })
_sym_db.RegisterMessage(FormatGlobalMemoryInfoRequest)

FormatGlobalMemoryInfoReply = _reflection.GeneratedProtocolMessageType('FormatGlobalMemoryInfoReply', (_message.Message,), {
  'DESCRIPTOR' : _FORMATGLOBALMEMORYINFOREPLY,
  '__module__' : 'src.ray.protobuf.node_manager_pb2'
  # @@protoc_insertion_point(class_scope:ray.rpc.FormatGlobalMemoryInfoReply)
  })
_sym_db.RegisterMessage(FormatGlobalMemoryInfoReply)

RequestObjectSpillageRequest = _reflection.GeneratedProtocolMessageType('RequestObjectSpillageRequest', (_message.Message,), {
  'DESCRIPTOR' : _REQUESTOBJECTSPILLAGEREQUEST,
  '__module__' : 'src.ray.protobuf.node_manager_pb2'
  # @@protoc_insertion_point(class_scope:ray.rpc.RequestObjectSpillageRequest)
  })
_sym_db.RegisterMessage(RequestObjectSpillageRequest)

RequestObjectSpillageReply = _reflection.GeneratedProtocolMessageType('RequestObjectSpillageReply', (_message.Message,), {
  'DESCRIPTOR' : _REQUESTOBJECTSPILLAGEREPLY,
  '__module__' : 'src.ray.protobuf.node_manager_pb2'
  # @@protoc_insertion_point(class_scope:ray.rpc.RequestObjectSpillageReply)
  })
_sym_db.RegisterMessage(RequestObjectSpillageReply)

ReleaseUnusedBundlesRequest = _reflection.GeneratedProtocolMessageType('ReleaseUnusedBundlesRequest', (_message.Message,), {
  'DESCRIPTOR' : _RELEASEUNUSEDBUNDLESREQUEST,
  '__module__' : 'src.ray.protobuf.node_manager_pb2'
  # @@protoc_insertion_point(class_scope:ray.rpc.ReleaseUnusedBundlesRequest)
  })
_sym_db.RegisterMessage(ReleaseUnusedBundlesRequest)

ReleaseUnusedBundlesReply = _reflection.GeneratedProtocolMessageType('ReleaseUnusedBundlesReply', (_message.Message,), {
  'DESCRIPTOR' : _RELEASEUNUSEDBUNDLESREPLY,
  '__module__' : 'src.ray.protobuf.node_manager_pb2'
  # @@protoc_insertion_point(class_scope:ray.rpc.ReleaseUnusedBundlesReply)
  })
_sym_db.RegisterMessage(ReleaseUnusedBundlesReply)



_NODEMANAGERSERVICE = _descriptor.ServiceDescriptor(
  name='NodeManagerService',
  full_name='ray.rpc.NodeManagerService',
  file=DESCRIPTOR,
  index=0,
  serialized_options=None,
  serialized_start=2041,
  serialized_end=3233,
  methods=[
  _descriptor.MethodDescriptor(
    name='RequestWorkerLease',
    full_name='ray.rpc.NodeManagerService.RequestWorkerLease',
    index=0,
    containing_service=None,
    input_type=_REQUESTWORKERLEASEREQUEST,
    output_type=_REQUESTWORKERLEASEREPLY,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='ReturnWorker',
    full_name='ray.rpc.NodeManagerService.ReturnWorker',
    index=1,
    containing_service=None,
    input_type=_RETURNWORKERREQUEST,
    output_type=_RETURNWORKERREPLY,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='ReleaseUnusedWorkers',
    full_name='ray.rpc.NodeManagerService.ReleaseUnusedWorkers',
    index=2,
    containing_service=None,
    input_type=_RELEASEUNUSEDWORKERSREQUEST,
    output_type=_RELEASEUNUSEDWORKERSREPLY,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='PrepareBundleResources',
    full_name='ray.rpc.NodeManagerService.PrepareBundleResources',
    index=3,
    containing_service=None,
    input_type=_PREPAREBUNDLERESOURCESREQUEST,
    output_type=_PREPAREBUNDLERESOURCESREPLY,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='CommitBundleResources',
    full_name='ray.rpc.NodeManagerService.CommitBundleResources',
    index=4,
    containing_service=None,
    input_type=_COMMITBUNDLERESOURCESREQUEST,
    output_type=_COMMITBUNDLERESOURCESREPLY,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='CancelResourceReserve',
    full_name='ray.rpc.NodeManagerService.CancelResourceReserve',
    index=5,
    containing_service=None,
    input_type=_CANCELRESOURCERESERVEREQUEST,
    output_type=_CANCELRESOURCERESERVEREPLY,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='CancelWorkerLease',
    full_name='ray.rpc.NodeManagerService.CancelWorkerLease',
    index=6,
    containing_service=None,
    input_type=_CANCELWORKERLEASEREQUEST,
    output_type=_CANCELWORKERLEASEREPLY,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='PinObjectIDs',
    full_name='ray.rpc.NodeManagerService.PinObjectIDs',
    index=7,
    containing_service=None,
    input_type=_PINOBJECTIDSREQUEST,
    output_type=_PINOBJECTIDSREPLY,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='GetNodeStats',
    full_name='ray.rpc.NodeManagerService.GetNodeStats',
    index=8,
    containing_service=None,
    input_type=_GETNODESTATSREQUEST,
    output_type=_GETNODESTATSREPLY,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='GlobalGC',
    full_name='ray.rpc.NodeManagerService.GlobalGC',
    index=9,
    containing_service=None,
    input_type=_GLOBALGCREQUEST,
    output_type=_GLOBALGCREPLY,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='FormatGlobalMemoryInfo',
    full_name='ray.rpc.NodeManagerService.FormatGlobalMemoryInfo',
    index=10,
    containing_service=None,
    input_type=_FORMATGLOBALMEMORYINFOREQUEST,
    output_type=_FORMATGLOBALMEMORYINFOREPLY,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='RequestObjectSpillage',
    full_name='ray.rpc.NodeManagerService.RequestObjectSpillage',
    index=11,
    containing_service=None,
    input_type=_REQUESTOBJECTSPILLAGEREQUEST,
    output_type=_REQUESTOBJECTSPILLAGEREPLY,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='ReleaseUnusedBundles',
    full_name='ray.rpc.NodeManagerService.ReleaseUnusedBundles',
    index=12,
    containing_service=None,
    input_type=_RELEASEUNUSEDBUNDLESREQUEST,
    output_type=_RELEASEUNUSEDBUNDLESREPLY,
    serialized_options=None,
  ),
])
_sym_db.RegisterServiceDescriptor(_NODEMANAGERSERVICE)

DESCRIPTOR.services_by_name['NodeManagerService'] = _NODEMANAGERSERVICE

# @@protoc_insertion_point(module_scope)



import ray
from ray.rllib.evaluation.postprocessing import compute_advantages, \
    Postprocessing
from ray.rllib.policy.sample_batch import SampleBatch
from ray.rllib.policy.torch_policy_template import build_torch_policy
from ray.rllib.utils.framework import try_import_torch

torch, nn = try_import_torch()


def actor_critic_loss(policy, model, dist_class, train_batch):
    logits, _ = model.from_batch(train_batch)
    values = model.value_function()
    dist = dist_class(logits, model)
    log_probs = dist.logp(train_batch[SampleBatch.ACTIONS])
    policy.entropy = dist.entropy().sum()
    policy.pi_err = -train_batch[Postprocessing.ADVANTAGES].dot(
        log_probs.reshape(-1))
    policy.value_err = torch.sum(
        torch.pow(
            values.reshape(-1) - train_batch[Postprocessing.VALUE_TARGETS],
            2.0))
    overall_err = sum([
        policy.pi_err,
        policy.config["vf_loss_coeff"] * policy.value_err,
        -policy.config["entropy_coeff"] * policy.entropy,
    ])
    return overall_err


def loss_and_entropy_stats(policy, train_batch):
    return {
        "policy_entropy": policy.entropy.item(),
        "policy_loss": policy.pi_err.item(),
        "vf_loss": policy.value_err.item(),
    }


def add_advantages(policy,
                   sample_batch,
                   other_agent_batches=None,
                   episode=None):

    completed = sample_batch[SampleBatch.DONES][-1]
    if completed:
        last_r = 0.0
    else:
        last_r = policy._value(sample_batch[SampleBatch.NEXT_OBS][-1])

    return compute_advantages(
        sample_batch, last_r, policy.config["gamma"], policy.config["lambda"],
        policy.config["use_gae"], policy.config["use_critic"])


def model_value_predictions(policy, input_dict, state_batches, model,
                            action_dist):
    return {SampleBatch.VF_PREDS: model.value_function()}


def apply_grad_clipping(policy, optimizer, loss):
    info = {}
    if policy.config["grad_clip"]:
        for param_group in optimizer.param_groups:
            # Make sure we only pass params with grad != None into torch
            # clip_grad_norm_. Would fail otherwise.
            params = list(
                filter(lambda p: p.grad is not None, param_group["params"]))
            if params:
                grad_gnorm = nn.utils.clip_grad_norm_(
                    params, policy.config["grad_clip"])
                if isinstance(grad_gnorm, torch.Tensor):
                    grad_gnorm = grad_gnorm.cpu().numpy()
                info["grad_gnorm"] = grad_gnorm
    return info


def torch_optimizer(policy, config):
    return torch.optim.Adam(policy.model.parameters(), lr=config["lr"])


class ValueNetworkMixin:
    def _value(self, obs):
        _ = self.model({"obs": torch.Tensor([obs]).to(self.device)}, [], [1])
        return self.model.value_function()[0]


A3CTorchPolicy = build_torch_policy(
    name="A3CTorchPolicy",
    get_default_config=lambda: ray.rllib.agents.a3c.a3c.DEFAULT_CONFIG,
    loss_fn=actor_critic_loss,
    stats_fn=loss_and_entropy_stats,
    postprocess_fn=add_advantages,
    extra_action_out_fn=model_value_predictions,
    extra_grad_process_fn=apply_grad_clipping,
    optimizer_fn=torch_optimizer,
    mixins=[ValueNetworkMixin],
)



import os
from typing import Optional, Tuple

from ray import services
from ray.autoscaler.sdk import rsync, configure_logging
from ray.tune.syncer import NodeSyncer
from ray.tune.sync_client import SyncClient
from ray.tune.utils import env_integer


class DockerSyncer(NodeSyncer):
    """DockerSyncer used for synchronization between Docker containers.
    This syncer extends the node syncer, but is usually instantiated
    without a custom sync client. The sync client defaults to
    ``DockerSyncClient`` instead.

    Set the env var `TUNE_SYNCER_VERBOSITY` to increase verbosity
    of syncing operations (0, 1, 2, 3). Defaults to 0.

    .. note::
        This syncer only works with the Ray cluster launcher.
        If you use your own Docker setup, make sure the nodes can connect
        to each other via SSH, and try the regular SSH-based syncer instead.

    Example:

    .. code-block:: python

        from ray.tune.integration.docker import DockerSyncer
        tune.run(train,
                 sync_config=tune.SyncConfig(
                     sync_to_driver=DockerSyncer))

    """

    _cluster_config_file = os.path.expanduser("~/ray_bootstrap_config.yaml")

    def __init__(self,
                 local_dir: str,
                 remote_dir: str,
                 sync_client: Optional[SyncClient] = None):
        configure_logging(
            log_style="record",
            verbosity=env_integer("TUNE_SYNCER_VERBOSITY", 0))
        self.local_ip = services.get_node_ip_address()
        self.worker_ip = None

        sync_client = sync_client or DockerSyncClient()
        sync_client.configure(self._cluster_config_file)

        super(NodeSyncer, self).__init__(local_dir, remote_dir, sync_client)

    def set_worker_ip(self, worker_ip: str):
        self.worker_ip = worker_ip

    @property
    def _remote_path(self) -> Tuple[str, str]:
        return (self.worker_ip, self._remote_dir)


class DockerSyncClient(SyncClient):
    """DockerSyncClient to be used by DockerSyncer.
    This client takes care of executing the synchronization
    commands for Docker nodes. In its ``sync_down`` and
    ``sync_up`` commands, it expects tuples for the source
    and target, respectively, for compatibility with docker.
    """

    def __init__(self):
        self._command_runners = {}
        self._cluster_config = None

    def configure(self, cluster_config_file: str):
        self._cluster_config_file = cluster_config_file

    def sync_up(self, source: str, target: Tuple[str, str]) -> bool:
        """Here target is a tuple (target_node, target_dir)"""
        target_node, target_dir = target

        # Add trailing slashes for rsync
        source = os.path.join(source, "")
        target_dir = os.path.join(target_dir, "")

        rsync(
            cluster_config=self._cluster_config_file,
            source=source,
            target=target_dir,
            down=False,
            ip_address=target_node,
            use_internal_ip=True)

        return True

    def sync_down(self, source: Tuple[str, str], target: str) -> bool:
        """Here source is a tuple (source_node, source_dir)"""
        source_node, source_dir = source

        # Add trailing slashes for rsync
        source_dir = os.path.join(source_dir, "")
        target = os.path.join(target, "")

        rsync(
            cluster_config=self._cluster_config_file,
            source=source_dir,
            target=target,
            down=True,
            ip_address=source_node,
            use_internal_ip=True)

        return True

    def delete(self, target: str) -> bool:
        raise NotImplementedError



from abc import ABCMeta, abstractmethod

from ray.serve.utils import logger


class AutoscalingPolicy:
    """Defines the interface for an autoscaling policy.

    To add a new autoscaling policy, a class should be defined that provides
    this interface. The class may be stateful, in which case it may also want
    to provide a non-default constructor. However, this state will be lost when
    the controller recovers from a failure.
    """
    __metaclass__ = ABCMeta

    def __init__(self, config):
        """Initialize the policy using the specified config dictionary."""
        self.config = config

    @abstractmethod
    def scale(self, router_queue_lens, curr_replicas):
        """Make a decision to scale backends.

        Arguments:
            router_queue_lens (Dict[str, int]): map of routers to their most
                recent queue length of unsent queries for this backend.
            curr_replicas (int): The number of replicas that the backend
                currently has.

        Returns:
            int The new number of replicas to scale this backend to.
        """
        return curr_replicas


class BasicAutoscalingPolicy(AutoscalingPolicy):
    """The default autoscaling policy based on basic thresholds for scaling.

    There is a minimum threshold for the average queue length in the cluster
    to scale up and a maximum threshold to scale down. Each period, a 'scale
    up' or 'scale down' decision is made. This decision must be made for a
    specified number of periods in a row before the number of replicas is
    actually scaled. See config options for more details.
    """

    def __init__(self, backend, config):
        self.backend = backend

        # The minimum number of replicas to scale down to.
        self.min_replicas = config.get("min_replicas", 1)
        # The maximum number of replicas to scale up to. -1 means there is no
        # limit.
        self.max_replicas = config.get("max_replicas", -1)
        if self.max_replicas == -1:
            self.max_replicas = float("inf")
        # The minimum average queue length to trigger scaling up.
        self.scale_up_threshold = config.get("scale_up_threshold", 5)
        # The maximum average queue length to trigger scaling down.
        self.scale_down_threshold = config.get("scale_down_threshold", 1)
        # The number of replicas to be added when scaling up.
        self.scale_up_num_replicas = config.get("scale_up_num_replicas", 2)
        # The number of replicas to be removed when scaling down.
        self.scale_down_num_replicas = config.get("scale_down_num_replicas", 1)
        # The number of consecutive 'scale up' decisions that need to be made
        # before the number of replicas is actually increased.
        self.scale_up_consecutive_periods = config.get(
            "scale_up_consecutive_periods", 2)
        # The number of consecutive 'scale down' decisions that need to be made
        # before the number of replicas is actually decreased.
        self.scale_down_consecutive_periods = config.get(
            "scale_down_consecutive_periods", 5)

        # Keeps track of previous decisions. Each time the load is above
        # 'scale_up_threshold', the counter is incremented and each time it is
        # below 'scale_down_threshold', the counter is decremented. When the
        # load is between the thresholds or a scaling decision is made, the
        # counter is reset to 0.
        self.decision_counter = 0

    def scale(self, router_queue_lens, curr_replicas):
        queue_lens = list(router_queue_lens.values())
        if len(queue_lens) == 0:
            return -1

        new_replicas = curr_replicas
        avg_queue_len = sum(queue_lens) / len(queue_lens)

        # Scale up.
        if avg_queue_len > self.scale_up_threshold:
            # If the previous decision was to scale down (the counter was
            # negative), we reset it and then increment it (set to 1).
            # Otherwise, just increment.
            if self.decision_counter < 0:
                self.decision_counter = 1
            else:
                self.decision_counter += 1

            # Only actually scale the replicas if we've made this decision for
            # 'scale_up_consecutive_periods' in a row.
            if (self.decision_counter >= self.scale_up_consecutive_periods
                    and curr_replicas < self.max_replicas):
                # TODO(edoakes): should we be resetting the counter here?
                self.decision_counter = 0
                new_replicas = min(self.max_replicas,
                                   curr_replicas + self.scale_up_num_replicas)
                logger.info("Increasing number of replicas for backend '{}' "
                            "from {} to {}".format(self.backend, curr_replicas,
                                                   new_replicas))

        # Scale down.
        elif avg_queue_len < self.scale_down_threshold:
            # If the previous decision was to scale up (the counter was
            # positive), reset it to zero before decrementing.
            if self.decision_counter > 0:
                self.decision_counter = -1
            else:
                self.decision_counter -= 1

            # Only actually scale the replicas if we've made this decision for
            # 'scale_down_consecutive_periods' in a row.
            if (self.decision_counter <=
                    -self.scale_down_consecutive_periods + 1
                    and curr_replicas > self.min_replicas):
                # TODO(edoakes): should we be resetting the counter here?
                self.decision_counter = 0
                new_replicas = max(
                    self.min_replicas,
                    curr_replicas - self.scale_down_num_replicas)
                logger.info("Decreasing number of replicas for backend '{}' "
                            "from {} to {}".format(self.backend, curr_replicas,
                                                   new_replicas))

        # Do nothing.
        else:
            self.decision_counter = 0

        return new_replicas



import os
import logging

import ray
import torch.distributed as dist
from ray.util.sgd.utils import find_free_port

logger = logging.getLogger(__name__)


def setup_address():
    ip = ray.services.get_node_ip_address()
    port = find_free_port()
    return f"tcp://{ip}:{port}"


def setup_process_group(url, world_rank, world_size, timeout, backend="gloo"):
    """Connects the distributed PyTorch backend.

    Args:
        url (str): the URL used to connect to distributed PyTorch.
        world_rank (int): the index of the runner.
        world_size (int): the total number of runners.
        timeout (timedelta): Timeout for operations executed against
            the process group.
        backend (str): One of gloo or nccl. Depending on
            build-time configuration
    """
    logger.debug("Connecting to {} world_rank: {} world_size: {}".format(
        url, world_rank, world_size))
    logger.debug(f"using {backend}")
    if backend == "nccl" and "NCCL_BLOCKING_WAIT" not in os.environ:
        logger.debug(
            "Setting NCCL_BLOCKING_WAIT for detecting node failure. "
            "To override this behavior, you can set NCCL_BLOCKING_WAIT=0.")
        os.environ["NCCL_BLOCKING_WAIT"] = "1"

    dist.init_process_group(
        backend=backend,
        init_method=url,
        rank=world_rank,
        world_size=world_size,
        timeout=timeout)



import numpy as np

from ray.rllib.policy.policy import Policy, LEARNER_STATS_KEY
from ray.rllib.policy.torch_policy import TorchPolicy
from ray.rllib.contrib.alpha_zero.core.mcts import Node, RootParentNode
from ray.rllib.utils.annotations import override
from ray.rllib.utils.framework import try_import_torch

torch, _ = try_import_torch()


class AlphaZeroPolicy(TorchPolicy):
    def __init__(self, observation_space, action_space, config, model, loss,
                 action_distribution_class, mcts_creator, env_creator,
                 **kwargs):
        super().__init__(
            observation_space,
            action_space,
            config,
            model=model,
            loss=loss,
            action_distribution_class=action_distribution_class)
        # we maintain an env copy in the policy that is used during mcts
        # simulations
        self.env_creator = env_creator
        self.mcts = mcts_creator()
        self.env = self.env_creator()
        self.env.reset()
        self.obs_space = observation_space

    @override(TorchPolicy)
    def compute_actions(self,
                        obs_batch,
                        state_batches=None,
                        prev_action_batch=None,
                        prev_reward_batch=None,
                        info_batch=None,
                        episodes=None,
                        **kwargs):

        input_dict = {"obs": obs_batch}
        if prev_action_batch:
            input_dict["prev_actions"] = prev_action_batch
        if prev_reward_batch:
            input_dict["prev_rewards"] = prev_reward_batch

        return self.compute_actions_from_input_dict(
            input_dict=input_dict,
            episodes=episodes,
            state_batches=state_batches,
        )

    @override(Policy)
    def compute_actions_from_input_dict(self,
                                        input_dict,
                                        explore=None,
                                        timestep=None,
                                        episodes=None,
                                        **kwargs):
        with torch.no_grad():
            actions = []
            for i, episode in enumerate(episodes):
                if episode.length == 0:
                    # if first time step of episode, get initial env state
                    env_state = episode.user_data["initial_state"]
                    # verify if env has been wrapped for ranked rewards
                    if self.env.__class__.__name__ == \
                            "RankedRewardsEnvWrapper":
                        # r2 env state contains also the rewards buffer state
                        env_state = {
                            "env_state": env_state,
                            "buffer_state": None
                        }
                    # create tree root node
                    obs = self.env.set_state(env_state)
                    tree_node = Node(
                        state=env_state,
                        obs=obs,
                        reward=0,
                        done=False,
                        action=None,
                        parent=RootParentNode(env=self.env),
                        mcts=self.mcts)
                else:
                    # otherwise get last root node from previous time step
                    tree_node = episode.user_data["tree_node"]

                # run monte carlo simulations to compute the actions
                # and record the tree
                mcts_policy, action, tree_node = self.mcts.compute_action(
                    tree_node)
                # record action
                actions.append(action)
                # store new node
                episode.user_data["tree_node"] = tree_node

                # store mcts policies vectors and current tree root node
                if episode.length == 0:
                    episode.user_data["mcts_policies"] = [mcts_policy]
                else:
                    episode.user_data["mcts_policies"].append(mcts_policy)

            return np.array(actions), [], self.extra_action_out(
                input_dict, kwargs.get("state_batches", []), self.model, None)

    @override(Policy)
    def postprocess_trajectory(self,
                               sample_batch,
                               other_agent_batches=None,
                               episode=None):
        # add mcts policies to sample batch
        sample_batch["mcts_policies"] = np.array(
            episode.user_data["mcts_policies"])[sample_batch["t"]]
        # final episode reward corresponds to the value (if not discounted)
        # for all transitions in episode
        final_reward = sample_batch["rewards"][-1]
        # if r2 is enabled, then add the reward to the buffer and normalize it
        if self.env.__class__.__name__ == "RankedRewardsEnvWrapper":
            self.env.r2_buffer.add_reward(final_reward)
            final_reward = self.env.r2_buffer.normalize(final_reward)
        sample_batch["value_label"] = final_reward * np.ones_like(
            sample_batch["t"])
        return sample_batch

    @override(Policy)
    def learn_on_batch(self, postprocessed_batch):
        train_batch = self._lazy_tensor_dict(postprocessed_batch)

        loss_out, policy_loss, value_loss = self._loss(
            self, self.model, self.dist_class, train_batch)
        self._optimizers[0].zero_grad()
        loss_out.backward()

        grad_process_info = self.extra_grad_process(self._optimizers[0],
                                                    loss_out)
        self._optimizers[0].step()

        grad_info = self.extra_grad_info(train_batch)
        grad_info.update(grad_process_info)
        grad_info.update({
            "total_loss": loss_out.detach().numpy(),
            "policy_loss": policy_loss.detach().numpy(),
            "value_loss": value_loss.detach().numpy()
        })

        return {LEARNER_STATS_KEY: grad_info}



from typing import Dict, List, Union
from ray import tune

import os


class TuneCallback:
    """Base class for Tune's XGBoost callbacks."""
    pass

    def __call__(self, env):
        raise NotImplementedError


class TuneReportCallback(TuneCallback):
    """XGBoost to Ray Tune reporting callback

    Reports metrics to Ray Tune.

    Args:
        metrics (str|list|dict): Metrics to report to Tune. If this is a list,
            each item describes the metric key reported to XGBoost,
            and it will reported under the same name to Tune. If this is a
            dict, each key will be the name reported to Tune and the respective
            value will be the metric key reported to XGBoost. If this is None,
            all metrics will be reported to Tune under their default names as
            obtained from XGBoost.

    Example:

    .. code-block:: python

        import xgboost
        from ray.tune.integration.xgboost import TuneReportCallback

        config = {
            # ...
            "eval_metric": ["auc", "logloss"]
        }

        # Report only log loss to Tune after each validation epoch:
        bst = xgb.train(
            config,
            train_set,
            evals=[(test_set, "eval")],
            verbose_eval=False,
            callbacks=[TuneReportCallback({"loss": "eval-logloss"})])

    """

    def __init__(self,
                 metrics: Union[None, str, List[str], Dict[str, str]] = None):
        if isinstance(metrics, str):
            metrics = [metrics]
        self._metrics = metrics

    def _get_report_dict(self, env):
        # Only one worker should report to Tune
        result_dict = dict(env.evaluation_result_list)
        if not self._metrics:
            report_dict = result_dict
        else:
            report_dict = {}
            for key in self._metrics:
                if isinstance(self._metrics, dict):
                    metric = self._metrics[key]
                else:
                    metric = key
                report_dict[key] = result_dict[metric]
        return report_dict

    def __call__(self, env):
        report_dict = self._get_report_dict(env)
        tune.report(**report_dict)


class _TuneCheckpointCallback(TuneCallback):
    """XGBoost checkpoint callback

    Saves checkpoints after each validation step.

    Checkpoint are currently not registered if no ``tune.report()`` call
    is made afterwards. Consider using ``TuneReportCheckpointCallback``
    instead.

    Args:
        filename (str): Filename of the checkpoint within the checkpoint
            directory. Defaults to "checkpoint".
        frequency (int): How often to save checkpoints. Per default, a
            checkpoint is saved every five iterations.

    """

    def __init__(self, filename: str = "checkpoint", frequency: int = 5):
        self._filename = filename
        self._frequency = frequency

    @staticmethod
    def _create_checkpoint(env, filename: str, frequency: int):
        if env.iteration % frequency > 0:
            return
        with tune.checkpoint_dir(step=env.iteration) as checkpoint_dir:
            env.model.save_model(os.path.join(checkpoint_dir, filename))

    def __call__(self, env):
        self._create_checkpoint(env, self._filename, self._frequency)


class TuneReportCheckpointCallback(TuneCallback):
    """XGBoost report and checkpoint callback

    Saves checkpoints after each validation step. Also reports metrics to Tune,
    which is needed for checkpoint registration.

    Args:
        metrics (str|list|dict): Metrics to report to Tune. If this is a list,
            each item describes the metric key reported to XGBoost,
            and it will reported under the same name to Tune. If this is a
            dict, each key will be the name reported to Tune and the respective
            value will be the metric key reported to XGBoost.
        filename (str): Filename of the checkpoint within the checkpoint
            directory. Defaults to "checkpoint". If this is None,
            all metrics will be reported to Tune under their default names as
            obtained from XGBoost.
        frequency (int): How often to save checkpoints. Per default, a
            checkpoint is saved every five iterations.

    Example:

    .. code-block:: python

        import xgboost
        from ray.tune.integration.xgboost import TuneReportCheckpointCallback

        config = {
            # ...
            "eval_metric": ["auc", "logloss"]
        }

        # Report only log loss to Tune after each validation epoch.
        # Save model as `xgboost.mdl`.
        bst = xgb.train(
            config,
            train_set,
            evals=[(test_set, "eval")],
            verbose_eval=False,
            callbacks=[TuneReportCheckpointCallback(
                {"loss": "eval-logloss"}, "xgboost.mdl)])

    """
    _checkpoint_callback_cls = _TuneCheckpointCallback
    _report_callbacks_cls = TuneReportCallback

    def __init__(self,
                 metrics: Union[None, str, List[str], Dict[str, str]] = None,
                 filename: str = "checkpoint",
                 frequency: int = 5):
        self._checkpoint = self._checkpoint_callback_cls(filename, frequency)
        self._report = self._report_callbacks_cls(metrics)

    def __call__(self, env):
        self._checkpoint(env)
        self._report(env)



"""Ray constants used in the Python code."""

import logging
import math
import os

logger = logging.getLogger(__name__)


def env_integer(key, default):
    if key in os.environ:
        return int(os.environ[key])
    return default


def env_bool(key, default):
    if key in os.environ:
        return True if os.environ[key].lower() == "true" else False
    return default


ID_SIZE = 20

# The default maximum number of bytes to allocate to the object store unless
# overridden by the user.
DEFAULT_OBJECT_STORE_MAX_MEMORY_BYTES = 200 * 10**9
# The default proportion of available memory allocated to the object store
DEFAULT_OBJECT_STORE_MEMORY_PROPORTION = 0.3
# The smallest cap on the memory used by the object store that we allow.
# This must be greater than MEMORY_RESOURCE_UNIT_BYTES * 0.7
OBJECT_STORE_MINIMUM_MEMORY_BYTES = 75 * 1024 * 1024
# The default maximum number of bytes that the non-primary Redis shards are
# allowed to use unless overridden by the user.
DEFAULT_REDIS_MAX_MEMORY_BYTES = 10**10
# The smallest cap on the memory used by Redis that we allow.
REDIS_MINIMUM_MEMORY_BYTES = 10**7
# If a user does not specify a port for the primary Ray service,
# we attempt to start the service running at this port.
DEFAULT_PORT = 6379

DEFAULT_DASHBOARD_IP = "127.0.0.1"
DEFAULT_DASHBOARD_PORT = 8265
PROMETHEUS_SERVICE_DISCOVERY_FILE = "prom_metrics_service_discovery.json"
# Default resource requirements for actors when no resource requirements are
# specified.
DEFAULT_ACTOR_METHOD_CPU_SIMPLE = 1
DEFAULT_ACTOR_CREATION_CPU_SIMPLE = 0
# Default resource requirements for actors when some resource requirements are
# specified in .
DEFAULT_ACTOR_METHOD_CPU_SPECIFIED = 0
DEFAULT_ACTOR_CREATION_CPU_SPECIFIED = 1
# Default number of return values for each actor method.
DEFAULT_ACTOR_METHOD_NUM_RETURN_VALS = 1

# If a remote function or actor (or some other export) has serialized size
# greater than this quantity, print an warning.
PICKLE_OBJECT_WARNING_SIZE = 10**7

# If remote functions with the same source are imported this many times, then
# print a warning.
DUPLICATE_REMOTE_FUNCTION_THRESHOLD = 100

# The maximum resource quantity that is allowed. TODO(rkn): This could be
# relaxed, but the current implementation of the node manager will be slower
# for large resource quantities due to bookkeeping of specific resource IDs.
MAX_RESOURCE_QUANTITY = 100000

# Each memory "resource" counts as this many bytes of memory.
MEMORY_RESOURCE_UNIT_BYTES = 50 * 1024 * 1024

# Number of units 1 resource can be subdivided into.
MIN_RESOURCE_GRANULARITY = 0.0001

# Fraction of plasma memory that can be reserved. It is actually 70% but this
# is set to 69% to leave some headroom.
PLASMA_RESERVABLE_MEMORY_FRACTION = 0.69


def round_to_memory_units(memory_bytes, round_up):
    """Round bytes to the nearest memory unit."""
    return from_memory_units(to_memory_units(memory_bytes, round_up))


def from_memory_units(memory_units):
    """Convert from memory units -> bytes."""
    return memory_units * MEMORY_RESOURCE_UNIT_BYTES


def to_memory_units(memory_bytes, round_up):
    """Convert from bytes -> memory units."""
    value = memory_bytes / MEMORY_RESOURCE_UNIT_BYTES
    if value < 1:
        raise ValueError(
            "The minimum amount of memory that can be requested is {} bytes, "
            "however {} bytes was asked.".format(MEMORY_RESOURCE_UNIT_BYTES,
                                                 memory_bytes))
    if isinstance(value, float) and not value.is_integer():
        # TODO(ekl) Ray currently does not support fractional resources when
        # the quantity is greater than one. We should fix memory resources to
        # be allocated in units of bytes and not 100MB.
        if round_up:
            value = int(math.ceil(value))
        else:
            value = int(math.floor(value))
    return int(value)


# Different types of Ray errors that can be pushed to the driver.
# TODO(rkn): These should be defined in flatbuffers and must be synced with
# the existing C++ definitions.
WAIT_FOR_CLASS_PUSH_ERROR = "wait_for_class"
PICKLING_LARGE_OBJECT_PUSH_ERROR = "pickling_large_object"
WAIT_FOR_FUNCTION_PUSH_ERROR = "wait_for_function"
TASK_PUSH_ERROR = "task"
REGISTER_REMOTE_FUNCTION_PUSH_ERROR = "register_remote_function"
FUNCTION_TO_RUN_PUSH_ERROR = "function_to_run"
VERSION_MISMATCH_PUSH_ERROR = "version_mismatch"
CHECKPOINT_PUSH_ERROR = "checkpoint"
REGISTER_ACTOR_PUSH_ERROR = "register_actor"
WORKER_CRASH_PUSH_ERROR = "worker_crash"
WORKER_DIED_PUSH_ERROR = "worker_died"
WORKER_POOL_LARGE_ERROR = "worker_pool_large"
PUT_RECONSTRUCTION_PUSH_ERROR = "put_reconstruction"
INFEASIBLE_TASK_ERROR = "infeasible_task"
RESOURCE_DEADLOCK_ERROR = "resource_deadlock"
REMOVED_NODE_ERROR = "node_removed"
MONITOR_DIED_ERROR = "monitor_died"
LOG_MONITOR_DIED_ERROR = "log_monitor_died"
REPORTER_DIED_ERROR = "reporter_died"
DASHBOARD_AGENT_DIED_ERROR = "dashboard_agent_died"
DASHBOARD_DIED_ERROR = "dashboard_died"
RAYLET_CONNECTION_ERROR = "raylet_connection_error"

# Used in gpu detection
RESOURCE_CONSTRAINT_PREFIX = "accelerator_type:"

RESOURCES_ENVIRONMENT_VARIABLE = "RAY_OVERRIDE_RESOURCES"

# The reporter will report its statistics this often (milliseconds).
REPORTER_UPDATE_INTERVAL_MS = env_integer("REPORTER_UPDATE_INTERVAL_MS", 2500)

# Number of attempts to ping the Redis server. See
# `services.py:wait_for_redis_to_start`.
START_REDIS_WAIT_RETRIES = env_integer("RAY_START_REDIS_WAIT_RETRIES", 12)

LOGGER_FORMAT = (
    "%(asctime)s\t%(levelname)s %(filename)s:%(lineno)s -- %(message)s")
LOGGER_FORMAT_HELP = f"The logging format. default='{LOGGER_FORMAT}'"
LOGGER_LEVEL = "info"
LOGGER_LEVEL_CHOICES = ["debug", "info", "warning", "error", "critical"]
LOGGER_LEVEL_HELP = ("The logging level threshold, choices=['debug', 'info',"
                     " 'warning', 'error', 'critical'], default='info'")
# Default param for RotatingFileHandler
# maxBytes. 10G by default. We intentionally set the default value high
# so that users who won't care don't know about the existence of this.
LOGGING_ROTATE_BYTES = 10 * 1000 * 1000 * 1000
# The default will grow logs up until 500GB without log loss.
LOGGING_ROTATE_BACKUP_COUNT = 50  # backupCount

# Constants used to define the different process types.
PROCESS_TYPE_REAPER = "reaper"
PROCESS_TYPE_MONITOR = "monitor"
PROCESS_TYPE_LOG_MONITOR = "log_monitor"
# TODO(sang): Delete it.
PROCESS_TYPE_REPORTER = "reporter"
PROCESS_TYPE_DASHBOARD = "dashboard"
PROCESS_TYPE_DASHBOARD_AGENT = "dashboard_agent"
PROCESS_TYPE_WORKER = "worker"
PROCESS_TYPE_RAYLET = "raylet"
PROCESS_TYPE_PLASMA_STORE = "plasma_store"
PROCESS_TYPE_REDIS_SERVER = "redis_server"
PROCESS_TYPE_WEB_UI = "web_ui"
PROCESS_TYPE_GCS_SERVER = "gcs_server"

# Log file names
MONITOR_LOG_FILE_NAME = f"{PROCESS_TYPE_MONITOR}.log"
LOG_MONITOR_LOG_FILE_NAME = f"{PROCESS_TYPE_LOG_MONITOR}.log"

WORKER_PROCESS_TYPE_IDLE_WORKER = "ray::IDLE"
WORKER_PROCESS_TYPE_SPILL_WORKER_NAME = "SpillWorker"
WORKER_PROCESS_TYPE_RESTORE_WORKER_NAME = "RestoreWorker"
WORKER_PROCESS_TYPE_SPILL_WORKER_IDLE = (
    f"ray::IDLE_{WORKER_PROCESS_TYPE_SPILL_WORKER_NAME}")
WORKER_PROCESS_TYPE_RESTORE_WORKER_IDLE = (
    f"ray::IDLE_{WORKER_PROCESS_TYPE_RESTORE_WORKER_NAME}")
WORKER_PROCESS_TYPE_SPILL_WORKER = (
    f"ray::SPILL_{WORKER_PROCESS_TYPE_SPILL_WORKER_NAME}")
WORKER_PROCESS_TYPE_RESTORE_WORKER = (
    f"ray::RESTORE_{WORKER_PROCESS_TYPE_RESTORE_WORKER_NAME}")
WORKER_PROCESS_TYPE_SPILL_WORKER_DELETE = (
    f"ray::DELETE_{WORKER_PROCESS_TYPE_SPILL_WORKER_NAME}")
WORKER_PROCESS_TYPE_RESTORE_WORKER_DELETE = (
    f"ray::DELETE_{WORKER_PROCESS_TYPE_RESTORE_WORKER_NAME}")

LOG_MONITOR_MAX_OPEN_FILES = 200

# The object metadata field uses the following format: It is a comma
# separated list of fields. The first field is mandatory and is the
# type of the object (see types below) or an integer, which is interpreted
# as an error value. The second part is optional and if present has the
# form DEBUG:<breakpoint_id>, it is used for implementing the debugger.

# A constant used as object metadata to indicate the object is cross language.
OBJECT_METADATA_TYPE_CROSS_LANGUAGE = b"XLANG"
# A constant used as object metadata to indicate the object is python specific.
OBJECT_METADATA_TYPE_PYTHON = b"PYTHON"
# A constant used as object metadata to indicate the object is raw bytes.
OBJECT_METADATA_TYPE_RAW = b"RAW"

# A constant used as object metadata to indicate the object is an actor handle.
# This value should be synchronized with the Java definition in
# ObjectSerializer.java
# TODO(fyrestone): Serialize the ActorHandle via the custom type feature
# of XLANG.
OBJECT_METADATA_TYPE_ACTOR_HANDLE = b"ACTOR_HANDLE"

# A constant indicating the debugging part of the metadata (see above).
OBJECT_METADATA_DEBUG_PREFIX = b"DEBUG:"

AUTOSCALER_RESOURCE_REQUEST_CHANNEL = b"autoscaler_resource_request"

# The default password to prevent redis port scanning attack.
# Hex for ray.
REDIS_DEFAULT_PASSWORD = "5241590000000000"

# The default ip address to bind to.
NODE_DEFAULT_IP = "127.0.0.1"

# The Mach kernel page size in bytes.
MACH_PAGE_SIZE_BYTES = 4096

# Max 64 bit integer value, which is needed to ensure against overflow
# in C++ when passing integer values cross-language.
MAX_INT64_VALUE = 9223372036854775807

# Object Spilling related constants
DEFAULT_OBJECT_PREFIX = "ray_spilled_object"
